-- v is the drawer
-- x, y are screen coordinates
-- text is the string to display
-- font is the patch prefix
-- flags is just like any other HUD function
-- alignment: -1 = right-aligned, 1 = left-aligned
-- color is a skincolor
local function DrawMotdString(v, x, y, scale, text, font, flags, alignment, color)
  local right
  local colormap = v.getColormap(0, color)
  local start
  local finish
  alignment = $ or 1
  color = $ or 0
  right = alignment < 0
  text = tostring(text)

  if right
    start = text:len()
    finish = 1
  else
    start = 1
    finish = text:len()
  end

  for i = start, finish, alignment
    local letter = font .. text:sub(i, i)
    if not v.patchExists(letter) then continue end

    local patch = v.cachePatch(letter)

    if right -- right aligned, change offset before drawing
      x = $ - patch.width*scale
    end

    v.drawScaled(x, y, scale, patch, flags, colormap)

    if not right -- left aligned, change offset after drawing
      x = $ + patch.width*scale
    end
  end
end

/*hud.add(function(v, p)
	v.drawString(316, 182, "Airboostheld: "+ (p.airboostheld), V_REDMAP, "right")
	v.drawString(316, 190, "Airboosted: "+ (p.airboost), V_REDMAP, "right")
end, "game")*/

local function modernboostlines(v, player)
	if player.mo and player.mo.skin == "modernsonic"
		if player.playerstate == PST_LIVE
			if (player.speed > FixedMul(65*FRACUNIT, player.mo.scale) and not (player.powers[pw_carry] == CR_MINECART) and not (player.powers[pw_sneakers]) and not (player.powers[pw_nocontrol]))
			or player.minecartboost == true
			or (player.customhoming and player.customhoming > 0)
				if player.boostlinetimer == 1
					v.drawScaled(0, 0, 65355, v.cachePatch("BSTFX1"), V_60TRANS|V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER, v.getColormap(nil, SKINCOLOR_SILVER))
				elseif player.boostlinetimer == 2
					v.drawScaled(320*FRACUNIT, 0, 65355, v.cachePatch("BSTFX1"), V_70TRANS|V_SNAPTORIGHT|V_SNAPTOTOP|V_FLIP|V_PERPLAYER, v.getColormap(nil, SKINCOLOR_SILVER))
				elseif player.boostlinetimer == 3
					v.drawScaled(0, 0, 65355, v.cachePatch("BSTFX2"), V_60TRANS|V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER, v.getColormap(nil, SKINCOLOR_SILVER))
				elseif player.boostlinetimer == 4
					v.drawScaled(320*FRACUNIT, 0, 65355, v.cachePatch("BSTFX2"), V_70TRANS|V_SNAPTORIGHT|V_SNAPTOTOP|V_FLIP|V_PERPLAYER, v.getColormap(nil, SKINCOLOR_SILVER))
				end
			end
		end
	end
end
hud.add(modernboostlines)

////////////////////////////////////////////////////////////////////////
--
-- Originally was Boss HUD rendering code.
-- (c) Inuyasha 2014.
-- Permissionwas granted by Inuyasha to use this code
-- and the associated graphics in other mods.
--
local posy = 0
local posx = 0
local posy2 = 0
local posx2 = 0
local client
local rushstary
local function modernhuddisable(v, player)
		if player.mo and player.mo.skin != "modernsonic"
		or player.hudstyle == "srb2"
		and not (G_RingSlingerGametype())
			if not hud.enabled("lives")
				hud.enable("lives")
			end
			if not hud.enabled("rings")
				hud.enable("rings")
			end
			if not hud.enabled("time")
				hud.enable("time")
			end
			if not hud.enabled("score")
				hud.enable("score")
			end
		elseif player.mo and player.mo.skin == "modernsonic"
			if not (G_RingSlingerGametype())
				if hud.enabled("time")
					hud.disable("time")
				end
				if hud.enabled("score")
					hud.disable("score")
				end
				if hud.enabled("lives")
					hud.disable("lives")
				end
				if hud.enabled("rings")
					hud.disable("rings")
				end
			else
				if not hud.enabled("rings")
					hud.enable("rings")
				end
				if not hud.enabled("time")
					hud.enable("time")
				end
				if not hud.enabled("score")
					hud.enable("score")
				end
			end
		end
	end
hud.add(modernhuddisable)
//Unleashed
local function unleashedhud(v, player)
	client = player
	local moderninflife = CV_FindVar("cooplives")
		if player.mo and player.mo.skin == "modernsonic"
        and not (player.powers[pw_carry] == CR_NIGHTSMODE)
		and player.hudstyle == "Unleashed"
		    local boostfill = 0;
			local boostcap = 0;
			local boostdrain = 0;
			if player.boostmeter == nil
				return
			end
			boostcap = $1 + 117
            boostfill = $1 + player.boostmeter
			boostdrain = $1 + player.boostmeter+1
 
			local p_ustartseg = v.cachePatch("UMTRENDL")
			local p_uonseg    = v.cachePatch("UMTRMIDL")
			local p_uoffseg   = v.cachePatch("UMTRMPTY")
			local p_uendseg   = v.cachePatch("UMTRENDR")
			local p_boost    = v.cachePatch("BOOST")
 
			posx = 186
			if G_RingSlingerGametype()
				posy = 160
			else
				posy = 174 
			end
			
			v.draw(posx, posy, p_uendseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
			posx = $1 - 1
			-- Step through backwards, to match the way we're drawing.
			for i = boostcap, 1, -1
					if boostfill >= i
						if player.boostmeter >= 117
							if not (player.powers[pw_super])
							and not (player.mysticsuper)
								v.draw(posx, posy, p_uonseg, V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_YELLOW))
							else
								v.draw(posx, posy, p_uonseg, V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
							end
						elseif player.boostmeter > 70
							v.draw(posx, posy, p_uonseg, V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_YELLOW))
						elseif player.boostmeter > 69
							v.draw(posx, posy, p_uonseg, V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_APRICOT", SKINCOLOR_APRICOT))
						elseif player.boostmeter > 30
							v.draw(posx, posy, p_uonseg, V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_ORANGE", SKINCOLOR_ORANGE))
						elseif player.boostmeter > 29
							v.draw(posx, posy, p_uonseg, V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_FLAME", SKINCOLOR_FLAME))
						elseif player.boostmeter > 0
							v.draw(posx, posy, p_uonseg, V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_RED", SKINCOLOR_RED))
						end
					else
                        v.draw(posx, posy, p_uoffseg, V_20TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					posx = $1 - 1;
					
			end
			-- Start segment
			v.draw(posx, posy, p_ustartseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);
			v.draw(posx+5, posy+5, v.cachePatch("ENERGY"), V_30TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
			v.draw(3, posy-4, v.cachePatch("SPEEDM00"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_PERPLAYER|V_HUDTRANS)
			
			if player.speed >= FRACUNIT
			and player.infntebst == 0
				if player.speed <= 3*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM01"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.speed <= 6*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM02"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.speed <= 9*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM03"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.speed <= 12*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM04"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.speed <= 15*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM05"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.speed <= 19*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM06"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.speed <= 21*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM07"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.speed <= 25*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM08"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.speed <= 30*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM09"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)		
				elseif player.speed <= 39*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM10"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.speed <= 46*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM11"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)		
				elseif player.speed <= 55*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM12"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.speed <= 70*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM13"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.speed >= 70*FRACUNIT
					v.draw(3, posy-4, v.cachePatch("SPEEDM14"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)	
				end
			end
			if player.infntebst > 0
				if player.infntebst < 25
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM01"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color)) 
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM01"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end
				elseif player.infntebst < 50
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM02"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM02"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end
				elseif player.infntebst < 75
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM03"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM03"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end
				elseif player.infntebst < 100
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM04"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM04"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end
				elseif player.infntebst < 125
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM05"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM05"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end
				elseif player.infntebst < 150
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM06"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM06"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end
				elseif player.infntebst < 175
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM07"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM07"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end
				elseif player.infntebst < 200
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM08"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM08"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end
				elseif player.infntebst < 225
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM09"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM09"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end
				elseif player.infntebst < 250
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM10"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM10"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end
				elseif player.infntebst < 275
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM11"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM11"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end	
				elseif player.infntebst < 300
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM12"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM12"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end
				elseif player.infntebst < 325
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM13"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM13"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end
				elseif player.infntebst >= 325
					if leveltime & 2
						v.draw(3, posy-4, v.cachePatch("RRUSHM14"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					else
						v.draw(3, posy-4, v.cachePatch("RRUSHM14"), V_10TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.rushcolor))
					end
				end
			end
			if player.infntebst == 0
				v.draw(posx-46, posy-3, v.cachePatch("SPEED"), V_30TRANS|V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
			else
				v.draw(posx-46, posy-3, v.cachePatch("URUSH"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_PERPLAYER|V_HUDTRANS)
			end
			if player.rings < 10
				DrawMotdString(v, 29*FRACUNIT, posy*FRACUNIT+12*FRACUNIT, FRACUNIT, "0", "UNLDHD1", V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				DrawMotdString(v, 37*FRACUNIT, posy*FRACUNIT+12*FRACUNIT, FRACUNIT, "0", "UNLDHD1", V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				DrawMotdString(v, 45*FRACUNIT, posy*FRACUNIT+12*FRACUNIT, FRACUNIT, player.rings, "UNLDHD1", V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				
			elseif player.rings < 100
				DrawMotdString(v, 29*FRACUNIT, posy*FRACUNIT+12*FRACUNIT, FRACUNIT, "0", "UNLDHD1", V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				DrawMotdString(v, 37*FRACUNIT, posy*FRACUNIT+12*FRACUNIT, FRACUNIT, player.rings, "UNLDHD1", V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
			else
				DrawMotdString(v, 29*FRACUNIT, posy*FRACUNIT+12*FRACUNIT, FRACUNIT, player.rings, "UNLDHD1", V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
			end
			if not (gamemap >= 60 and gamemap <= 66)
			and not (G_RingSlingerGametype())
				--Life
				if player.powers[pw_super]
					v.drawScaled((10*FRACUNIT), (8*FRACUNIT), FRACUNIT/2, v.cachePatch("ULIFE2"), V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS, v.getColormap(player.mo.skin, player.mo.color))
				else
					v.drawScaled((10*FRACUNIT), (8*FRACUNIT), FRACUNIT/2, v.cachePatch("ULIFE1"), V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS, v.getColormap(player.mo.skin, player.mo.color))
				end
				if moderninflife.value ~= 0
				and player.lives
				and player.lives ~= INFLIVES
					if player.lives < 10
						DrawMotdString(v, 33*FRACUNIT, 12*FRACUNIT, FRACUNIT, "0", "UNLDHD1", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 41*FRACUNIT, 12*FRACUNIT, FRACUNIT, player.lives, "UNLDHD1", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 33*FRACUNIT, 12*FRACUNIT, FRACUNIT, player.lives, "UNLDHD1", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
				else
					v.drawScaled((33*FRACUNIT), (12*FRACUNIT), FRACUNIT, v.cachePatch("UNLDHDI"), V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
				end
				--Time
				v.drawScaled((0), (30*FRACUNIT), FRACUNIT, v.cachePatch("UTIME"), V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
				local minutes = min(99, G_TicsToMinutes(player.realtime))
				if G_TicsToMinutes(player.realtime) < 10
					DrawMotdString(v, 14*FRACUNIT, 40*FRACUNIT, FRACUNIT, "0", "UNLDHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					DrawMotdString(v, 21*FRACUNIT, 40*FRACUNIT, FRACUNIT, minutes, "UNLDHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				else
					DrawMotdString(v, 13*FRACUNIT, 40*FRACUNIT, FRACUNIT, minutes, "UNLDHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				end
				if G_TicsToSeconds(player.realtime) < 10
					DrawMotdString(v, 36*FRACUNIT, 40*FRACUNIT, FRACUNIT, "0", "UNLDHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					DrawMotdString(v, 43*FRACUNIT, 40*FRACUNIT, FRACUNIT, G_TicsToSeconds(player.realtime), "UNLDHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				else
					DrawMotdString(v, 35*FRACUNIT, 40*FRACUNIT, FRACUNIT, G_TicsToSeconds(player.realtime), "UNLDHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				end
				if G_TicsToCentiseconds(player.realtime) < 10
					DrawMotdString(v, 57*FRACUNIT, 40*FRACUNIT, FRACUNIT, "0", "UNLDHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					DrawMotdString(v, 64*FRACUNIT, 40*FRACUNIT, FRACUNIT, G_TicsToCentiseconds(player.realtime), "UNLDHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				else
					DrawMotdString(v, 57*FRACUNIT, 40*FRACUNIT, FRACUNIT, G_TicsToCentiseconds(player.realtime), "UNLDHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				end
				v.drawScaled((51*FRACUNIT), (41*FRACUNIT), FRACUNIT, v.cachePatch("UNLDHDS"), V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
				v.drawScaled((30*FRACUNIT), (41*FRACUNIT), FRACUNIT, v.cachePatch("UNLDHDS"), V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
			//	v.drawString(51, 41, ":", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				//v.drawString(30, 41, ":", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				--Score
				v.drawScaled((0), (60*FRACUNIT), FRACUNIT, v.cachePatch("USCORE"), V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
				local pscore = player.score
				local offx = 71
				local zerocount = 9
				
				while (pscore > 0) do
					pscore = $ / 10
					offx = $ - 7
					//if (zerocount == 8) then offx = $ + 1 end --tiny little offset hack
					zerocount = $ - 1
				end
				
				local zstring = "000000000"
				
					DrawMotdString(v, 8*FRACUNIT, 70*FRACUNIT, FRACUNIT, zstring:sub(1,zerocount), "UNLDHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				if (player.score) then
					DrawMotdString(v, offx*FRACUNIT, 70*FRACUNIT, FRACUNIT, player.score, "UNLDHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				end
			end
			if player.hudring
				if player.hudring > 5
					v.drawScaled((posx*FRACUNIT-58*FRACUNIT), (posy*FRACUNIT+5*FRACUNIT), FRACUNIT+FRACUNIT/10, v.cachePatch("RINGC"), V_10TRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER|V_HUDTRANS)
				elseif player.hudring > 4
					v.drawScaled((posx*FRACUNIT-58*FRACUNIT-FRACUNIT/4), (posy*FRACUNIT+5*FRACUNIT), FRACUNIT+FRACUNIT/8, v.cachePatch("RINGC"), V_20TRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER|V_HUDTRANS)
				elseif player.hudring > 3
					v.drawScaled((posx*FRACUNIT-58*FRACUNIT-FRACUNIT/2), (posy*FRACUNIT+5*FRACUNIT), FRACUNIT+FRACUNIT/6, v.cachePatch("RINGC"), V_30TRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER|V_HUDTRANS)
				elseif player.hudring > 2
					v.drawScaled((posx*FRACUNIT-59*FRACUNIT-FRACUNIT/4), (posy*FRACUNIT+4*FRACUNIT), FRACUNIT+FRACUNIT/4, v.cachePatch("RINGC"), V_40TRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER|V_HUDTRANS)
				elseif player.hudring > 1
					v.drawScaled((posx*FRACUNIT-59*FRACUNIT-FRACUNIT/2), (posy*FRACUNIT+4*FRACUNIT-FRACUNIT/2), FRACUNIT+FRACUNIT/3, v.cachePatch("RINGC"), V_50TRANS|V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER|V_HUDTRANS)
				end
			end
		end
	end
hud.add(unleashedhud)
//Generations
local function generationshud(v, player)
	client = player
	local moderninflife = CV_FindVar("cooplives")
		if player.mo and player.mo.skin == "modernsonic"
        and not (player.powers[pw_carry] == CR_NIGHTSMODE)
		and player.hudstyle == "Generations"
		    local boostfill = 0;
			local boostcap = 0;
			local boostdrain = 0;
			local rushfill = 0;
			local rushcap = 0;
			local rushdrain = 0;
			if player.boostmeter == nil
				return
			end
			boostcap = $1 + 117
            boostfill = $1 + player.boostmeter
			boostdrain = $1 + player.boostmeter+1
			rushcap = $1 + 10*TICRATE
            rushfill = $1 + player.infntebst
			rushdrain = $1 + player.infntebst+1
 
			local p_gstartseg = v.cachePatch("GMTRENDL")
			local p_gstartsega = v.cachePatch("GMTRENDA")
			local p_gonseg    = v.cachePatch("GMTRMIDL")
			local p_goffseg   = v.cachePatch("GMTRMPTY")
			local p_glife    = v.cachePatch("GLIFE")
			local p_gring    = v.cachePatch("GRING")
			local p_gtime    = v.cachePatch("GTIME")
			local p_gcolon	 = v.cachePatch("GENSHDS")	
			local p_gperiod	 = v.cachePatch("GENSHDP")	
			posx = 150
			posx2 = 150
			if G_RingSlingerGametype()
				posy = 160
				posy2 = 160
			else
				posy = 174 
				posy2 = 174
			end
			
			posx = $1 - 1
			posx2 = $1 - 1
			-- Step through backwards, to match the way we're drawing.
			for i = boostcap, 1, -1
				if boostfill >= i
					if player.powers[pw_super]
						v.draw(posx, posy, p_gonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
					elseif player.boostmeter > 50
						v.draw(posx, posy, p_gonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_EMERALD))
					elseif player.boostmeter > 49
						v.draw(posx, posy, p_gonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_APRICOT", SKINCOLOR_GREEN))
					elseif player.boostmeter > 20
						v.draw(posx, posy, p_gonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_ORANGE", SKINCOLOR_YELLOW))
					elseif player.boostmeter > 19
						v.draw(posx, posy, p_gonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_FLAME", SKINCOLOR_ORANGE))
					elseif player.boostmeter > 0
						v.draw(posx, posy, p_gonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_RED", SKINCOLOR_RED))
					end
				else
					v.draw(posx, posy, p_goffseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				end
				posx = $1 - 1;
			end
			if player.infntebst > 0
				for i = rushcap, 1, -3
					if rushfill >= i
						if player.flashingtimer >= 8 and player.flashingtimer <= 9
							v.draw(posx2, posy2, p_gonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_SUPERSKY5))
						elseif player.flashingtimer >= 6 and player.flashingtimer <= 7
						or player.flashingtimer >= 10 and player.flashingtimer <= 11
							v.draw(posx2, posy2, p_gonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_SUPERSKY4))
						elseif player.flashingtimer >= 4 and player.flashingtimer <= 5
						or player.flashingtimer >= 12 and player.flashingtimer <= 13
							v.draw(posx2, posy2, p_gonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_SUPERSKY3))
						elseif player.flashingtimer >= 2 and player.flashingtimer <= 3
						or player.flashingtimer >= 14 and player.flashingtimer <= 15
							v.draw(posx2, posy2, p_gonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_SUPERSKY2))
						elseif player.flashingtimer >= 0 and player.flashingtimer <= 1
							v.draw(posx2, posy2, p_gonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_SUPERSKY1))
						end
						
					end
					posx2 = $1 - 1;
				end
			end
			-- Start segment
			if player.actionswap == true
				v.draw(posx, posy, p_gstartsega, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);
			else
				v.draw(posx, posy, p_gstartseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);
			end
			if not (gamemap >= 60 and gamemap <= 66)
			and not (G_RingSlingerGametype())
				--Ring
					if player.rings < 10
						DrawMotdString(v, 48*FRACUNIT, 50*FRACUNIT, FRACUNIT/3, player.rings, "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 40*FRACUNIT, 50*FRACUNIT, FRACUNIT/3, "0", "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 32*FRACUNIT, 50*FRACUNIT, FRACUNIT/3, "0", "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					elseif player.rings < 100
						DrawMotdString(v, 40*FRACUNIT, 50*FRACUNIT, FRACUNIT/3, player.rings, "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 32*FRACUNIT, 50*FRACUNIT, FRACUNIT/3, "0", "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 32*FRACUNIT, 50*FRACUNIT, FRACUNIT/3, player.rings, "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					v.drawScaled((10*FRACUNIT), (45*FRACUNIT), 3*FRACUNIT/4, p_gring, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					
				--Life
					v.drawScaled((270*FRACUNIT), (15*FRACUNIT), 3*FRACUNIT/4, p_glife, V_SNAPTORIGHT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_NONE", player.mo.color))
					if moderninflife.value ~= 0
					and player.lives
					and player.lives ~= INFLIVES
						if player.lives < 10
							DrawMotdString(v, 301*FRACUNIT, 20*FRACUNIT, FRACUNIT/3, player.lives, "GENSHD", V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS)
							DrawMotdString(v, 293*FRACUNIT, 20*FRACUNIT, FRACUNIT/3, "0", "GENSHD", V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS)
						else
							DrawMotdString(v, 293*FRACUNIT, 20*FRACUNIT, FRACUNIT/3, player.lives, "GENSHD", V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS)
						end
					else
						v.drawScaled((294*FRACUNIT), (18*FRACUNIT), FRACUNIT, v.cachePatch("GENSHDI"), V_SNAPTORIGHT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					end
				--Time
					v.drawScaled((10*FRACUNIT), (15*FRACUNIT), 3*FRACUNIT/4, p_gtime, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					local minutes = min(99, G_TicsToMinutes(player.realtime))
					if G_TicsToMinutes(player.realtime) < 10
						DrawMotdString(v, 32*FRACUNIT, 20*FRACUNIT, FRACUNIT/3, "0", "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 40*FRACUNIT, 20*FRACUNIT, FRACUNIT/3, minutes, "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 32*FRACUNIT, 20*FRACUNIT, FRACUNIT/3, minutes, "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					if G_TicsToSeconds(player.realtime) < 10
						DrawMotdString(v, 56*FRACUNIT, 20*FRACUNIT, FRACUNIT/3, "0", "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 64*FRACUNIT, 20*FRACUNIT, FRACUNIT/3, G_TicsToSeconds(player.realtime), "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 56*FRACUNIT, 20*FRACUNIT, FRACUNIT/3, G_TicsToSeconds(player.realtime), "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					if G_TicsToCentiseconds(player.realtime) < 10
						DrawMotdString(v, 80*FRACUNIT, 23*FRACUNIT, FRACUNIT/4, "0", "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 88*FRACUNIT, 23*FRACUNIT, FRACUNIT/4, G_TicsToCentiseconds(player.realtime), "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 80*FRACUNIT, 23*FRACUNIT, FRACUNIT/4, G_TicsToCentiseconds(player.realtime), "GENSHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					v.drawScaled(50*FRACUNIT, 21*FRACUNIT, FRACUNIT/3, p_gcolon, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					v.drawScaled(74*FRACUNIT, 26*FRACUNIT, FRACUNIT/3, p_gperiod, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				
			end
		end
	end
hud.add(generationshud)

//Forces
local function forceshud(v, player)
	client = player
	local moderninflife = CV_FindVar("cooplives")
		if player.mo and player.mo.skin == "modernsonic"
        and not (player.powers[pw_carry] == CR_NIGHTSMODE)
		and player.hudstyle == "Forces"
		    local boostfill = 0;
			local boostcap = 0;
			local boostdrain = 0;
			if player.boostmeter == nil
				return
			end
			boostcap = $1 + 117
            boostfill = $1 + player.boostmeter
			boostdrain = $1 + player.boostmeter+1
 
			local p_fstartseg = v.cachePatch("FMTRENDL")
			local p_fonseg    = v.cachePatch("FMTRMIDL")
			local p_foffseg   = v.cachePatch("FMTRMPTY")
			local p_fendseg   = v.cachePatch("FMTRENDR")
			local p_flife    = v.cachePatch("ULIFE")
			local p_fscore    = v.cachePatch("FSCORE")
			local p_fring    = v.cachePatch("FRING")
			local p_ftime    = v.cachePatch("FTIME")
			local p_fcolon	 = v.cachePatch("FORCHDS")	
			local p_fperiod	 = v.cachePatch("FORCHDP")	
			local p_fbost	 = v.cachePatch("FORCBST")
			posx = 130
			if G_RingSlingerGametype()
				posy = 155
			else
				posy = 170
			end
			
			v.draw(posx, posy, p_fendseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
			posx = $1 - 1
			-- Step through backwards, to match the way we're drawing.
			for i = boostcap, 1, -1
					if boostfill >= i
						if player.boostmeter >= 117
							if not (player.powers[pw_super])
							and not (player.mysticsuper)
								if leveltime & 2
									v.draw(posx, posy, p_fonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.mo.color))
								else
									v.draw(posx, posy, p_fonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.rushcolor))
								end
							else
								v.draw(posx, posy, p_fonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
							end
						else
							v.draw(posx, posy, p_fonseg,V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_WAVE))
						end
					else
                        v.draw(posx, posy, p_foffseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						
					end
					posx = $1 - 1;
			end
			-- Start segment
			v.draw(posx, posy, p_fstartseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);
			if player.boosting == true
			or player.airboost == true
			and player.cmd.buttons & BT_CUSTOM1
			or player.superboost == true
			or player.minecartboost == true
				if player.boostmeter >= 117
					if leveltime & 3
						if leveltime & 2
							v.drawScaled(posx*FRACUNIT+boostfill*FRACUNIT+FRACUNIT, posy*FRACUNIT-8*FRACUNIT, 3*FRACUNIT/4, p_fbost, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE",player.mo.color))
						else
							v.drawScaled(posx*FRACUNIT+boostfill*FRACUNIT+FRACUNIT, posy*FRACUNIT-8*FRACUNIT, 3*FRACUNIT/4, p_fbost, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE",player.rushcolor))
						end
					else
						if leveltime & 2
							v.drawScaled(posx*FRACUNIT+boostfill*FRACUNIT+FRACUNIT, posy*FRACUNIT-10*FRACUNIT, 18*FRACUNIT/20, p_fbost, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE",player.mo.color))
						else
							v.drawScaled(posx*FRACUNIT+boostfill*FRACUNIT+FRACUNIT, posy*FRACUNIT-10*FRACUNIT, 18*FRACUNIT/20, p_fbost, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE",player.rushcolor))
						end
					end
				else
					if leveltime & 2
						v.drawScaled(posx*FRACUNIT+boostfill*FRACUNIT+FRACUNIT, posy*FRACUNIT-8*FRACUNIT, 3*FRACUNIT/4, p_fbost, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE",SKINCOLOR_SAPPHIRE))
					else
						v.drawScaled(posx*FRACUNIT+boostfill*FRACUNIT+FRACUNIT, posy*FRACUNIT-10*FRACUNIT, 18*FRACUNIT/20, p_fbost, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE",SKINCOLOR_SAPPHIRE))
					end
					
				end
			end
			if not (gamemap >= 60 and gamemap <= 66)
			and not (G_RingSlingerGametype())
				--Ring
					v.drawScaled((0*FRACUNIT), (0*FRACUNIT), FRACUNIT, p_fring, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					if player.rings < 10
						DrawMotdString(v, 75*FRACUNIT, 7*FRACUNIT, FRACUNIT, player.rings, "FORCHD1", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					elseif player.rings < 100
						DrawMotdString(v, 68*FRACUNIT, 7*FRACUNIT, FRACUNIT, player.rings, "FORCHD1", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					elseif player.rings < 1000
						DrawMotdString(v, 57*FRACUNIT, 7*FRACUNIT, FRACUNIT, player.rings, "FORCHD1", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 46*FRACUNIT, 7*FRACUNIT, FRACUNIT, player.rings, "FORCHD1", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					
					
				--Life
					if moderninflife.value ~= 0
					and player.lives
					and player.lives ~= INFLIVES
						if player.powers[pw_super]
							v.drawScaled((255*FRACUNIT), (5*FRACUNIT), 3*FRACUNIT/4, v.cachePatch("ULIFE2"), V_SNAPTORIGHT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS, v.getColormap(player.mo.skin, player.mo.color))
						else
							v.drawScaled((255*FRACUNIT), (5*FRACUNIT), 3*FRACUNIT/4, v.cachePatch("ULIFE1"), V_SNAPTORIGHT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS, v.getColormap(player.mo.skin, player.mo.color))
						end
						if player.lives < 10
							DrawMotdString(v, 292*FRACUNIT, 12*FRACUNIT, FRACUNIT, player.lives, "FORCHD2", V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS)
							DrawMotdString(v, 285*FRACUNIT, 12*FRACUNIT, FRACUNIT, "0", "FORCHD2", V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS)
						else
							DrawMotdString(v, 282*FRACUNIT, 12*FRACUNIT, FRACUNIT, player.lives, "FORCHD2", V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS)
						end
					end
					
				--Time
					v.drawScaled((13*FRACUNIT), (33*FRACUNIT), FRACUNIT, p_ftime, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					local minutes = min(99, G_TicsToMinutes(player.realtime))
					if G_TicsToMinutes(player.realtime) < 10
						//v.drawString(14, 41, "0", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						//v.drawString(22, 41, minutes, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 58*FRACUNIT, 33*FRACUNIT, FRACUNIT, "0", "FORCHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 65*FRACUNIT, 33*FRACUNIT, FRACUNIT, minutes, "FORCHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 58*FRACUNIT, 33*FRACUNIT, FRACUNIT, minutes, "FORCHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					if G_TicsToSeconds(player.realtime) < 10
						DrawMotdString(v, 78*FRACUNIT, 33*FRACUNIT, FRACUNIT, "0", "FORCHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 85*FRACUNIT, 33*FRACUNIT, FRACUNIT, G_TicsToSeconds(player.realtime), "FORCHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 78*FRACUNIT, 33*FRACUNIT, FRACUNIT, G_TicsToSeconds(player.realtime), "FORCHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					if G_TicsToCentiseconds(player.realtime) < 10
						DrawMotdString(v, 98*FRACUNIT, 33*FRACUNIT, FRACUNIT, "0", "FORCHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 105*FRACUNIT, 33*FRACUNIT, FRACUNIT, G_TicsToCentiseconds(player.realtime), "FORCHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 98*FRACUNIT, 33*FRACUNIT, FRACUNIT, G_TicsToCentiseconds(player.realtime), "FORCHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					v.drawScaled(73*FRACUNIT, 34*FRACUNIT, FRACUNIT, p_fcolon, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					v.drawScaled(93*FRACUNIT, 38*FRACUNIT, FRACUNIT, p_fperiod, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				--Score
					v.drawScaled((12*FRACUNIT), (53*FRACUNIT), FRACUNIT, p_fscore, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					local pscore = player.score
					local offx = 112*FRACUNIT
					local zerocount = 1
					
					while (pscore > 0) do
						pscore = $ / 10
						offx = $ - 7*FRACUNIT
						if (zerocount == 1) then offx = $ + FRACUNIT end --tiny little offset hack
						zerocount = $ - 1
					end
					
					local zstring = "0"
					DrawMotdString(v, 105*FRACUNIT, 53*FRACUNIT, FRACUNIT, zstring:sub(1,zerocount), "FORCHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					//v.drawString(1, 71, zstring:sub(1,zerocount),  V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					if (player.score) then
						DrawMotdString(v, offx, 53*FRACUNIT, FRACUNIT, player.score, "FORCHD2", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						//v.drawString(offx, 71, player.score, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
			end
		end
	end
hud.add(forceshud)
//Colors
local function colorshud(v, player)
	client = player
	local moderninflife = CV_FindVar("cooplives")
		if player.mo and player.mo.skin == "modernsonic"
        and not (player.powers[pw_carry] == CR_NIGHTSMODE)
		and player.hudstyle == "Colors"
			//Boost Meter
		    local boostfill = 0;
			local boostcap = 0;
			local boostdrain = 0;
			if player.boostmeter == nil
				return
			end
			boostcap = $1 + 117
            boostfill = $1 + player.boostmeter
			boostdrain = $1 + player.boostmeter+1
			//Rush Meter
			local rushfill = 0;
			local rushcap = 0;
			local rushdrain = 0;
			if player.infntebst == nil
				return
			end
			rushcap = $1 + 10*TICRATE
            rushfill = $1 + player.infntebst
			rushdrain = $1 + player.infntebst+1
 
			local p_cstartseg = v.cachePatch("CMTRENDL")
			local p_cstartseg2 = v.cachePatch("CMTRSS")
			local p_cstartrush = v.cachePatch("CMTRENDS")
			local p_conseg    = v.cachePatch("CMTRMIDL")
			local p_conseg2    = v.cachePatch("CMTRMIDS")
			local p_coffseg   = v.cachePatch("CMTRMPTY")
			local p_coffseg2   = v.cachePatch("CMTRMPTS")
			local p_cendseg   = v.cachePatch("CMTRENDR")
			local p_clife    = v.cachePatch("CLIFE")
			local p_clifes    = v.cachePatch("CLIFES")
			local p_cscore    = v.cachePatch("CSCORE")
			local p_cring    = v.cachePatch("CRING")
			local p_ctime    = v.cachePatch("CTIME")
			local p_ccolon	 = v.cachePatch("COLRHDS")	
			local p_cperiod	 = v.cachePatch("COLRHDP")	
			local p_cbost	 = v.cachePatch("COLRBST")
			local p_cemerald1	 = v.cachePatch("CEMERLD1")
			local p_cemerald2	 = v.cachePatch("CEMERLD2")
			local p_cemerald3	 = v.cachePatch("CEMERLD3")
			local p_cemerald4	 = v.cachePatch("CEMERLD4")
			local p_cemerald5	 = v.cachePatch("CEMERLD5")
			//Without Rush
			posx = 170
			if G_RingSlingerGametype()
				posy = 160
			else
				posy = 170
			end
			
			v.draw(posx, posy, p_cendseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
			posx = $1 - 1
			-- Step through backwards, to match the way we're drawing.
			for i = boostcap, 1, -1
					if boostfill >= i
						if player.boostmeter >= 117
							v.draw(posx, posy, p_conseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
						else
							v.draw(posx, posy, p_conseg,V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_RASPBERRY))
						end
					else
                        v.draw(posx, posy, p_coffseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						
					end
					posx = $1 - 1;
			end
			//With Rush
			if player.infntebst > 0
				posx2 = 116
				if G_RingSlingerGametype()
					posy2 = 143
				else
					posy2 = 153
				end
				posx2 = $1 - 1
				-- Step through backwards, to match the way we're drawing.
				for i = rushcap, 1, -6
					if rushfill >= i
						if leveltime & 2
							v.draw(posx2, posy2, p_conseg2, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.mo.color))
						else
							v.draw(posx2, posy2, p_conseg2, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.rushcolor))
						end
					else
						v.draw(posx2, posy2, p_coffseg2, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						
					end
					posx2 = $1 - 1;
				end
			end
			-- Start segment 
			if player.infntebst > 0
				v.draw(posx, posy, p_cstartrush, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);
			end
			if All7Emeralds(emeralds)
			and player.rings >= 50
			or player.powers[pw_super]
				v.draw(posx, posy, p_cstartseg2, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);	
				v.draw(posx, posy, p_cemerald1, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);
			else
				v.draw(posx, posy, p_cstartseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);
			end
			if player.powers[pw_super]
				if player.flashingtimer >= 8 and player.flashingtimer <= 9
					v.draw(posx, posy, p_cemerald5, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);
				elseif player.flashingtimer >= 6 and player.flashingtimer <= 7
				or player.flashingtimer >= 10 and player.flashingtimer <= 11
					v.draw(posx, posy, p_cemerald4, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);
				elseif player.flashingtimer >= 4 and player.flashingtimer <= 5
				or player.flashingtimer >= 12 and player.flashingtimer <= 13
					v.draw(posx, posy, p_cemerald3, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);
				elseif player.flashingtimer >= 2 and player.flashingtimer <= 3
				or player.flashingtimer >= 14 and player.flashingtimer <= 15
					v.draw(posx, posy, p_cemerald2, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);
				elseif player.flashingtimer >= 0 and player.flashingtimer <= 1
					v.draw(posx, posy, p_cemerald1, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);
				end
			end
			if not (gamemap >= 60 and gamemap <= 66)
			and not (G_RingSlingerGametype())
				--Ring
					v.drawScaled((17*FRACUNIT), (20*FRACUNIT), FRACUNIT, p_cring, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					if player.rings < 10
						DrawMotdString(v, 47*FRACUNIT, 29*FRACUNIT, FRACUNIT, "0", "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 55*FRACUNIT, 29*FRACUNIT, FRACUNIT, "0", "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 63*FRACUNIT, 29*FRACUNIT, FRACUNIT, "0", "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 71*FRACUNIT, 29*FRACUNIT, FRACUNIT, player.rings, "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					elseif player.rings < 100
						DrawMotdString(v, 47*FRACUNIT, 29*FRACUNIT, FRACUNIT, "0", "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 55*FRACUNIT, 29*FRACUNIT, FRACUNIT, "0", "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 63*FRACUNIT, 29*FRACUNIT, FRACUNIT, player.rings, "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					elseif player.rings < 1000
						DrawMotdString(v, 47*FRACUNIT, 29*FRACUNIT, FRACUNIT, "0", "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 55*FRACUNIT, 29*FRACUNIT, FRACUNIT, player.rings, "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 47*FRACUNIT, 29*FRACUNIT, FRACUNIT, player.rings, "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					
					
				--Life
					if player.powers[pw_super]
						v.drawScaled((17*FRACUNIT), (3*FRACUNIT), FRACUNIT, p_clifes, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_NONE", player.mo.color))
					else
						v.drawScaled((17*FRACUNIT), (3*FRACUNIT), FRACUNIT, p_clife, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_NONE", player.mo.color))
					end
					if moderninflife.value ~= 0
					and player.lives
					and player.lives ~= INFLIVES
						if player.lives < 10
							DrawMotdString(v, 55*FRACUNIT, 12*FRACUNIT, FRACUNIT, player.lives, "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
							DrawMotdString(v, 47*FRACUNIT, 12*FRACUNIT, FRACUNIT, "0", "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						else
							DrawMotdString(v, 47*FRACUNIT, 12*FRACUNIT, FRACUNIT, player.lives, "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						end
					else
						v.drawScaled((50*FRACUNIT), (25*FRACUNIT/2), FRACUNIT, v.cachePatch("COLRHDI"), V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					end
				--Time
					v.drawScaled((17*FRACUNIT), (36*FRACUNIT), FRACUNIT, p_ctime, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					local minutes = min(99, G_TicsToMinutes(player.realtime))
					if G_TicsToMinutes(player.realtime) < 10
						//v.drawString(14, 41, "0", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						//v.drawString(22, 41, minutes, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 47*FRACUNIT, 46*FRACUNIT, FRACUNIT, "0", "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 55*FRACUNIT, 46*FRACUNIT, FRACUNIT, minutes, "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 47*FRACUNIT, 46*FRACUNIT, FRACUNIT, minutes, "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					if G_TicsToSeconds(player.realtime) < 10
						DrawMotdString(v, 67*FRACUNIT, 46*FRACUNIT, FRACUNIT, "0", "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 75*FRACUNIT, 46*FRACUNIT, FRACUNIT, G_TicsToSeconds(player.realtime), "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 67*FRACUNIT, 46*FRACUNIT, FRACUNIT, G_TicsToSeconds(player.realtime), "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					if G_TicsToCentiseconds(player.realtime) < 10
						DrawMotdString(v, 87*FRACUNIT, 46*FRACUNIT, FRACUNIT, "0", "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 95*FRACUNIT, 46*FRACUNIT, FRACUNIT, G_TicsToCentiseconds(player.realtime), "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 87*FRACUNIT, 46*FRACUNIT, FRACUNIT, G_TicsToCentiseconds(player.realtime), "COLRHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					v.drawScaled(63*FRACUNIT, 47*FRACUNIT, FRACUNIT, p_ccolon, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					v.drawScaled(82*FRACUNIT, 51*FRACUNIT, FRACUNIT, p_cperiod, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					
				--Score
					v.drawScaled((240*FRACUNIT), (14*FRACUNIT), FRACUNIT, p_cscore, V_SNAPTORIGHT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					local pscore = player.score
					local offx = 306
					local zerocount = 8
					
					while (pscore > 0) do
						pscore = $ / 10
						offx = $ - 8
						if (zerocount == 1) then offx = $ + FRACUNIT end --tiny little offset hack
						zerocount = $ - 1
					end
					
					local zstring = "000000000"
					DrawMotdString(v, 242*FRACUNIT, 28*FRACUNIT, FRACUNIT, zstring:sub(1,zerocount), "COLRHD", V_SNAPTOTOP|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS)
					//v.drawString(1, 71, zstring:sub(1,zerocount),  V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					if (player.score) then
						DrawMotdString(v, offx*FRACUNIT, 28*FRACUNIT, FRACUNIT, player.score, "COLRHD", V_SNAPTORIGHT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
						//v.drawString(offx, 71, player.score, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
			end
		end
	end
hud.add(colorshud)
//06
local function nextgenhud(v, player)
	client = player
	local moderninflife = CV_FindVar("cooplives")
		if player.mo and player.mo.skin == "modernsonic"
        and not (player.powers[pw_carry] == CR_NIGHTSMODE)
		and player.hudstyle == "06"
		    local boostfill = 0;
			local boostcap = 0;
			local boostdrain = 0;
			if player.boostmeter == nil
				return
			end
			boostcap = $1 + 117
            boostfill = $1 + player.boostmeter
			boostdrain = $1 + player.boostmeter+1
 
			local p_nstartseg = v.cachePatch("NMTRENDL")
			local p_nonseg    = v.cachePatch("NMTRMIDL")
			local p_noffseg   = v.cachePatch("NMTRMPTY")
			local p_nendseg   = v.cachePatch("NMTRENDR")
			local p_nlifen    = v.cachePatch("NLIFEN")
			local p_nlifes    = v.cachePatch("NLIFES")
			local p_nscore    = v.cachePatch("NSCORE")
			local p_nring    = v.cachePatch("NRING")
			local p_ntime    = v.cachePatch("NTIME")
			local p_ncolon	 = v.cachePatch("NGENHDS")	
			local p_nperiod	 = v.cachePatch("NGENHDP")	
			local p_nbost	 = v.cachePatch("NGENBST")
			local p_ngem	 = v.cachePatch("NGEM")
			posx = 130
			if G_RingSlingerGametype()
				posy = 155
			else
				posy = 174 
			end
			
			v.draw(posx, posy, p_nendseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
			posx = $1 - 1
			v.draw(posx, posy, p_nstartseg, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS);
			if player.infntebst > 6*TICRATE
				if leveltime & 2
					v.drawScaled(291*FRACUNIT, 185*FRACUNIT, FRACUNIT, p_ngem, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.mo.color))
					v.drawScaled(303*FRACUNIT, 185*FRACUNIT, FRACUNIT, p_ngem, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.mo.color))
					v.drawScaled(297*FRACUNIT, 185*FRACUNIT, FRACUNIT, p_ngem, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.mo.color))
				else
					v.drawScaled(303*FRACUNIT, 185*FRACUNIT, FRACUNIT, p_ngem, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.rushcolor))
					v.drawScaled(291*FRACUNIT, 185*FRACUNIT, FRACUNIT, p_ngem, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.rushcolor))
					v.drawScaled(297*FRACUNIT, 185*FRACUNIT, FRACUNIT, p_ngem, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.rushcolor))
				end
			elseif player.infntebst > 3*TICRATE
				if leveltime & 2
					v.drawScaled(291*FRACUNIT, 185*FRACUNIT, FRACUNIT, p_ngem, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.mo.color))
					v.drawScaled(297*FRACUNIT, 185*FRACUNIT, FRACUNIT, p_ngem, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.mo.color))
				else
					v.drawScaled(291*FRACUNIT, 185*FRACUNIT, FRACUNIT, p_ngem, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.rushcolor))
					v.drawScaled(297*FRACUNIT, 185*FRACUNIT, FRACUNIT, p_ngem, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.rushcolor))
				end
			elseif player.infntebst > 0
				if leveltime & 2
					v.drawScaled(291*FRACUNIT, 185*FRACUNIT, FRACUNIT, p_ngem, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.mo.color))
				else
					v.drawScaled(291*FRACUNIT, 185*FRACUNIT, FRACUNIT, p_ngem, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.rushcolor))
				end
			end
			-- Step through backwards, to match the way we're drawing.
			for i = boostcap, 1, -1
					if boostfill >= i
						if player.boostmeter >= 117
							if not (player.powers[pw_super])
							and not (player.mysticsuper)
								if leveltime & 2
									v.draw(posx, posy, p_nonseg, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.mo.color))
								else
									v.draw(posx, posy, p_nonseg, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.rushcolor))
								end
							else
								v.draw(posx, posy, p_nonseg, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
							end
						else
							v.draw(posx, posy, p_nonseg,V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_SKY))
						end
					else
                        v.draw(posx, posy, p_noffseg, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER|V_HUDTRANS)
						
					end
					posx = $1 - 1;
			end
			-- Start segment
		//	v.draw(posx, posy, p_nstartseg, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER);
			if not (gamemap >= 60 and gamemap <= 66)
			and not (G_RingSlingerGametype())
				--Ring
					v.drawScaled((0*FRACUNIT), (38*FRACUNIT), FRACUNIT/2, p_nring, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					if player.rings < 10
						DrawMotdString(v, 53*FRACUNIT, 44*FRACUNIT, FRACUNIT/2, "0", "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 60*FRACUNIT, 44*FRACUNIT, FRACUNIT/2, "0", "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 67*FRACUNIT, 44*FRACUNIT, FRACUNIT/2, player.rings, "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					elseif player.rings < 100
						DrawMotdString(v, 53*FRACUNIT, 44*FRACUNIT, FRACUNIT/2, "0", "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 60*FRACUNIT, 44*FRACUNIT, FRACUNIT/2, player.rings, "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 53*FRACUNIT, 44*FRACUNIT, FRACUNIT/2, player.rings, "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					
					
				--Life
					if player.powers[pw_super]
						v.drawScaled((0*FRACUNIT), (56*FRACUNIT), FRACUNIT/2, p_nlifes, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_NONE", player.mo.color))
					else
						v.drawScaled((0*FRACUNIT), (56*FRACUNIT), FRACUNIT/2, p_nlifen, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_NONE", player.mo.color))
					end
					if moderninflife.value ~= 0
					and player.lives
					and player.lives ~= INFLIVES
						if player.lives < 10
							DrawMotdString(v, 67*FRACUNIT, 62*FRACUNIT, FRACUNIT/2, player.lives, "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
							DrawMotdString(v, 60*FRACUNIT, 62*FRACUNIT, FRACUNIT/2, "0", "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						else
							DrawMotdString(v, 60*FRACUNIT, 62*FRACUNIT, FRACUNIT/2, player.lives, "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						end
					else
						v.drawScaled((60*FRACUNIT), (62*FRACUNIT), FRACUNIT/2, v.cachePatch("NGENHDI"), V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					end
				--Time
					v.drawScaled((0*FRACUNIT), (23*FRACUNIT), FRACUNIT/2, p_ntime, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					local minutes = min(99, G_TicsToMinutes(player.realtime))
					if G_TicsToMinutes(player.realtime) < 10
						//v.drawString(14, 41, "0", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						//v.drawString(22, 41, minutes, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 45*FRACUNIT, 27*FRACUNIT, FRACUNIT/2, "0", "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 52*FRACUNIT, 27*FRACUNIT, FRACUNIT/2, minutes, "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 45*FRACUNIT, 27*FRACUNIT, FRACUNIT/2, minutes, "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					if G_TicsToSeconds(player.realtime) < 10
						DrawMotdString(v, 65*FRACUNIT, 27*FRACUNIT, FRACUNIT/2, "0", "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 72*FRACUNIT, 27*FRACUNIT, FRACUNIT/2, G_TicsToSeconds(player.realtime), "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 65*FRACUNIT, 27*FRACUNIT, FRACUNIT/2, G_TicsToSeconds(player.realtime), "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					if G_TicsToCentiseconds(player.realtime) < 10
						DrawMotdString(v, 87*FRACUNIT, 27*FRACUNIT, FRACUNIT/2, "0", "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 94*FRACUNIT, 27*FRACUNIT, FRACUNIT/2, G_TicsToCentiseconds(player.realtime), "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 87*FRACUNIT, 27*FRACUNIT, FRACUNIT/2, G_TicsToCentiseconds(player.realtime), "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					v.drawScaled(60*FRACUNIT, 27*FRACUNIT, FRACUNIT/2, p_ncolon, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					v.drawScaled(79*FRACUNIT, 27*FRACUNIT, FRACUNIT/2, p_nperiod, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					
				--Score
					v.drawScaled((0*FRACUNIT), (5*FRACUNIT), FRACUNIT/2, p_nscore, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					local pscore = player.score
					local offx = 101*FRACUNIT
					local zerocount = 1
					
					while (pscore > 0) do
						pscore = $ / 10
						offx = $ - 7*FRACUNIT
						if (zerocount == 1) then offx = $ + FRACUNIT end --tiny little offset hack
						zerocount = $ - 1
					end
					
					local zstring = "0"
					DrawMotdString(v, 93*FRACUNIT, 9*FRACUNIT, FRACUNIT/2, zstring:sub(1,zerocount), "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					//v.drawString(1, 71, zstring:sub(1,zerocount),  V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					if (player.score) then
						DrawMotdString(v, offx, 9*FRACUNIT, FRACUNIT/2, player.score, "NGENHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						//v.drawString(offx, 71, player.score, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
			end
		end
	end
hud.add(nextgenhud)

//rush
local function rushhud(v, player)
	client = player
	local moderninflife = CV_FindVar("cooplives")
		if player.mo and player.mo.skin == "modernsonic"
        and not (player.powers[pw_carry] == CR_NIGHTSMODE)
		and player.hudstyle == "Rush"
		    local boostfill1 = 0;
			local boostfill2 = 0;
			local boostfill3 = 0;
			local boostcap1 = 0;
			local boostcap2 = 0;
			local boostcap3 = 0;
			if player.boostmeter == nil
				return
			end
			boostcap1 = $1 + 39
			boostcap2 = $1 + 39
			boostcap3 = $1 + 39
            boostfill1 = $1 - player.boostmeter+39
			boostfill2 = $1 - player.boostmeter+78
			boostfill3 = $1 - player.boostmeter+117
 
			local p_ronseg    = v.cachePatch("RMTRMIDL")
			local p_roffseg   = v.cachePatch("RMTRMPTY")
			local p_rlifen    = v.cachePatch("RLIFEN")
			local p_rlifes    = v.cachePatch("RLIFES")
			local p_rring    = v.cachePatch("RRING")
			local p_rtime    = v.cachePatch("RTIME")
			local p_rcolon	 = v.cachePatch("RUSHHDS")
			posx = 10
			if G_RingSlingerGametype()
				posy = 180
				rushstary = 180
			else
				posy = 152 
				rushstary = 152
			end
			
			posx = $1 - 1
			//v.draw(posx, posy, p_rstartseg, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER);
			-- Step through backwards, to match the way we're drawing.
			if player.boostmeter <= 39
				for i = boostcap1, 1, -2
						if boostfill1 >= i
							v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_roffseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", SKINCOLOR_WHITE))
						else
							if player.boostmeter >= 117
								if not (player.powers[pw_super])
								and not (player.mysticsuper)
									if leveltime & 2
										v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_ronseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", SKINCOLOR_YELLOW))
									else
										v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_ronseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", SKINCOLOR_RED))
									end
								else
									v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_ronseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
								end
							else
								v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_ronseg,V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_CYAN))
								
							end
							
						end
						posy = $1 - 5;
				end
			elseif player.boostmeter <= 78
				for i = boostcap2, 1, -2
						if boostfill2 >= i
							v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_roffseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", SKINCOLOR_CYAN))
						else
							if player.boostmeter >= 117
								if not (player.powers[pw_super])
								and not (player.mysticsuper)
									if leveltime & 2
										v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_ronseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", SKINCOLOR_YELLOW))
									else
										v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_ronseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", SKINCOLOR_RED))
									end
								else
									v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_ronseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
								end
							else
								v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_ronseg,V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_YELLOW))
								
							end
							
						end
						posy = $1 - 5;
				end
			else
				for i = boostcap3, 1, -2
						if boostfill3 >= i
							v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_roffseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", SKINCOLOR_YELLOW))
						else
							if player.boostmeter >= 117
								if not (player.powers[pw_super])
								and not (player.mysticsuper)
									if leveltime & 2
										v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_ronseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", SKINCOLOR_YELLOW))
									else
										v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_ronseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", SKINCOLOR_RED))
									end
								else
									v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_ronseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
								end
							else
								v.drawScaled(posx*FRACUNIT, posy*FRACUNIT, 3*FRACUNIT/4, p_ronseg,V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_RED))
							end
							
						end
						posy = $1 - 5;
				end
			end
			--Rush Star
			if player.boostmeter <= 39
				v.draw(posx+1, rushstary-115, v.cachePatch("RSHSTRA1"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
			elseif player.boostmeter <= 78
				if player.startimerone >= 0 and player.startimerone <= 3
					v.draw(posx+1, rushstary-115, v.cachePatch("RSHSTRA1"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.startimerone > 3 and player.startimerone <= 8
				or player.startimerone > 9 and player.startimerone <= 13
					v.draw(posx, rushstary-116, v.cachePatch("RSHSTRA2"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.startimerone > 6 and player.startimerone <= 9
					v.draw(posx+1, rushstary-115, v.cachePatch("RSHSTRA3"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				end
			elseif player.boostmeter < 117
				if player.startimertwo >= 0 and player.startimertwo <= 3
					v.draw(posx-1, rushstary-117, v.cachePatch("RSHSTRB1"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.startimertwo > 3 and player.startimertwo <= 6
					v.draw(posx, rushstary-116, v.cachePatch("RSHSTRB2"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.startimertwo > 6 and player.startimertwo <= 9
					v.draw(posx, rushstary-116, v.cachePatch("RSHSTRB3"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.startimertwo > 9 and player.startimertwo <= 12
					v.draw(posx, rushstary-116, v.cachePatch("RSHSTRB4"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.startimertwo > 12 and player.startimertwo <= 15
					v.draw(posx-1, rushstary-117, v.cachePatch("RSHSTRB5"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.startimertwo > 15 and player.startimertwo <= 19
					v.draw(posx-1, rushstary-117, v.cachePatch("RSHSTRB6"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				end
			elseif player.boostmeter >= 117
				if player.startimerthree >= 0 and player.startimerthree <= 8
					v.draw(posx-1, rushstary-117, v.cachePatch("RSHSTRC1"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.startimerthree > 8 and player.startimerthree <= 16
					v.draw(posx-1, rushstary-123, v.cachePatch("RSHSTRC2"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.startimerthree > 16 and player.startimerthree <= 24
					v.draw(posx+1, rushstary-123, v.cachePatch("RSHSTRC3"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.startimerthree > 24 and player.startimerthree <= 32
					v.draw(posx-2, rushstary-122, v.cachePatch("RSHSTRC4"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.startimerthree > 32 and player.startimerthree <= 40
					v.draw(posx-4, rushstary-121, v.cachePatch("RSHSTRC5"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.startimerthree > 40 and player.startimerthree <= 48
					v.draw(posx-3, rushstary-123, v.cachePatch("RSHSTRC6"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				elseif player.startimerthree > 48 //and player.startimerthree <= 43
					v.draw(posx, rushstary-121, v.cachePatch("RSHSTRC7"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				end
			end
			-- Start segment
			if not (gamemap >= 60 and gamemap <= 66)
			and not (G_RingSlingerGametype())
				--Ring
					v.drawScaled((7*FRACUNIT), (0), FRACUNIT, p_rring, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					if player.rings < 10
						DrawMotdString(v, 28*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, "0", "RUSHHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 38*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, "0", "RUSHHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 48*FRACUNIT,15*FRACUNIT/2, FRACUNIT, player.rings, "RUSHHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					elseif player.rings < 100
						DrawMotdString(v, 28*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, "0", "RUSHHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 38*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, player.rings, "RUSHHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 28*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, player.rings, "RUSHHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					
					
				--Life
					if player.powers[pw_super]
						v.drawScaled((8*FRACUNIT), (170*FRACUNIT), FRACUNIT, p_rlifes, V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_NONE", player.mo.color))
					else
						v.drawScaled((8*FRACUNIT), (170*FRACUNIT), FRACUNIT, p_rlifen, V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_NONE", player.mo.color))
					end
					if moderninflife.value ~= 0
					and player.lives
					and player.lives ~= INFLIVES
						if player.lives < 10
							DrawMotdString(v, 48*FRACUNIT, 180*FRACUNIT, FRACUNIT, player.lives, "RUSHHD", V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
							DrawMotdString(v, 38*FRACUNIT, 180*FRACUNIT, FRACUNIT, "0", "RUSHHD", V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						else
							DrawMotdString(v, 38*FRACUNIT, 180*FRACUNIT, FRACUNIT, player.lives, "RUSHHD", V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						end
					else
						v.drawScaled((38*FRACUNIT), (182*FRACUNIT), FRACUNIT, v.cachePatch("RUSHHDI"), V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER|V_HUDTRANS)
					end
				--Time
					v.drawScaled((114*FRACUNIT), (5*FRACUNIT), FRACUNIT, p_rtime, V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					local minutes = min(99, G_TicsToMinutes(player.realtime))
					if G_TicsToMinutes(player.realtime) < 10
						//v.drawString(14, 41, "0", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						//v.drawString(22, 41, minutes, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 132*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, "0", "RUSHHD", V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 142*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, minutes, "RUSHHD", V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 132*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, minutes, "RUSHHD", V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					end
					if G_TicsToSeconds(player.realtime) < 10
						DrawMotdString(v, 160*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, "0", "RUSHHD", V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 170*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, G_TicsToSeconds(player.realtime), "RUSHHD", V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 160*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, G_TicsToSeconds(player.realtime), "RUSHHD", V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					end
					if G_TicsToCentiseconds(player.realtime) < 10
						DrawMotdString(v, 188*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, "0", "RUSHHD", V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
						DrawMotdString(v, 198*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, G_TicsToCentiseconds(player.realtime), "RUSHHD", V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					else
						DrawMotdString(v, 188*FRACUNIT, 15*FRACUNIT/2, FRACUNIT, G_TicsToCentiseconds(player.realtime), "RUSHHD", V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					end
					v.drawScaled(153*FRACUNIT, 9*FRACUNIT, FRACUNIT, p_rcolon, V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					v.drawScaled(181*FRACUNIT, 9*FRACUNIT, FRACUNIT, p_rcolon, V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					
				--Score
					/*v.drawScaled((0*FRACUNIT), (5*FRACUNIT), FRACUNIT/2, p_rscore, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					local pscore = player.score
					local offx = 101*FRACUNIT
					local zerocount = 1
					
					while (pscore > 0) do
						pscore = $ / 10
						offx = $ - 7*FRACUNIT
						if (zerocount == 1) then offx = $ + FRACUNIT end --tiny little offset hack
						zerocount = $ - 1
					end
					
					local zstring = "0"
					DrawMotdString(v, 93*FRACUNIT, 9*FRACUNIT, FRACUNIT/2, zstring:sub(1,zerocount), "RUSHHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					if (player.score) then
						DrawMotdString(v, offx, 9*FRACUNIT, FRACUNIT/2, player.score, "RUSHHD", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end*/
			end
		end
	end
hud.add(rushhud)
//SRB2
local function defaulthud(v, player)
	client = player
	local moderninflife = CV_FindVar("cooplives")
		if player.mo and player.mo.skin == "modernsonic"
        and not (player.powers[pw_carry] == CR_NIGHTSMODE)
		and player.hudstyle == "srb2"
		    local boostfill = 0;
			local boostcap = 0;
			local boostdrain = 0;
			if player.boostmeter == nil
				return
			end
			boostcap = $1 + 117
            boostfill = $1 + player.boostmeter
			boostdrain = $1 + player.boostmeter+1
 
			local p_sstartseg = v.cachePatch("SMTRENDL")
			local p_sonseg    = v.cachePatch("SMTRMIDL")
			local p_soffseg   = v.cachePatch("SMTRMPTY")
			local p_sendseg   = v.cachePatch("SMTRENDR")
			posx = 135
			if G_RingSlingerGametype()
				posy = 160
			else
				posy = 170
			end
			
			v.draw(posx, posy, p_sendseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
			posx = $1 - 1
			-- Step through backwards, to match the way we're drawing.
			for i = boostcap, 1, -1
					if boostfill >= i
						if player.boostmeter >= 117
							if not (player.powers[pw_super])
							and not (player.mysticsuper)
								if leveltime & 2
									v.draw(posx, posy, p_sonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.mo.color))
								else
									v.draw(posx, posy, p_sonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", player.rushcolor))
								end
							else
								v.draw(posx, posy, p_sonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", player.mo.color))
							end
						else
							if (player.boosting == true or (player.airboost == true and player.boostheld == true))
							and (((player.cmd.buttons & BT_USE) and player.actionswap == true) or ((player.cmd.buttons & BT_CUSTOM1) and player.actionswap == false))
								if leveltime & 2
									v.draw(posx, posy, p_sonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", SKINCOLOR_RED))
								else
									v.draw(posx, posy, p_sonseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_WAVE", SKINCOLOR_BONE))
								end
							else
								v.draw(posx, posy, p_sonseg,V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_YELLOW", SKINCOLOR_EMERALD))	
							end
						end
					else
                        v.draw(posx, posy, p_soffseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
						
					end
					posx = $1 - 1;
			end
			-- Start segment
			v.draw(posx, posy, p_sstartseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS); 
		end
	end
hud.add(defaulthud)

//SRB2Gens
local function shithud(v, player)
	client = player
	local moderninflife = CV_FindVar("cooplives")
		if player.mo and player.mo.skin == "modernsonic"
        and not (player.powers[pw_carry] == CR_NIGHTSMODE)
		and player.hudstyle == "Shit"
			local p_gstartseg = v.cachePatch("BMTRENDL")
			local p_gstartsega = v.cachePatch("BMTRENDA")
			local p_gonseg    = v.cachePatch("BMTRMIDL")
			local p_goffseg   = v.cachePatch("BMTRMPTY")
			local p_glife    = v.cachePatch("BLIFE")
			local p_glife2   = v.cachePatch("BLIFE2")
			local p_gscore   = v.cachePatch("BSCORE")
			local p_gring    = v.cachePatch("BRING")
			local p_gring2    = v.cachePatch("BRING2")
			local p_gtime    = v.cachePatch("BTIME")
			local p_gcolon	 = v.cachePatch("BCOLON")	
			posx = 40
			if G_RingSlingerGametype()
				posy = 170
			else
				posy = 190 
			end
			
			-- Start segment
			v.draw(posx, posy, p_gstartseg, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS);
			if not (gamemap >= 60 and gamemap <= 66)
			and not (G_RingSlingerGametype())
				--Ring
					v.drawNum(58, 56, player.rings, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					if player.rings == 0
						if leveltime & 2
							v.drawScaled((10*FRACUNIT), (53*FRACUNIT), 3*FRACUNIT/4, p_gring, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
						else
							v.drawScaled((10*FRACUNIT), (53*FRACUNIT), 3*FRACUNIT/4, p_gring2, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
						end
					else
						v.drawScaled((10*FRACUNIT), (53*FRACUNIT), 3*FRACUNIT/4, p_gring, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					end
				--Life
					v.drawScaled((270*FRACUNIT), (10*FRACUNIT), 3*FRACUNIT/4, p_glife, V_SNAPTORIGHT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_NONE", player.mo.color))
					v.drawScaled((18*FRACUNIT), (180*FRACUNIT), FRACUNIT/2, p_glife2, V_SNAPTOLEFT|V_SNAPTOBOTTOM|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_NONE", player.mo.color))
					if moderninflife.value ~= 0
					and player.lives
					and player.lives ~= INFLIVES
						v.drawNum(55, 183, player.lives, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					else
						v.drawScaled((294*FRACUNIT), (18*FRACUNIT), FRACUNIT, v.cachePatch("GENSHDI"), V_SNAPTORIGHT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					end
				--Time
					v.drawScaled((12*FRACUNIT), (30*FRACUNIT), 3*FRACUNIT/4, p_gtime, V_SNAPTOLEFT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS)
					local minutes = min(99, G_TicsToMinutes(player.realtime))
					v.drawNum(40, 33, minutes, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					if G_TicsToSeconds(player.realtime) < 10
						v.drawNum(56, 33, "0", V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					end
					v.drawNum(64, 33, G_TicsToSeconds(player.realtime), V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					v.draw(40, 33, p_gcolon, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
				--Score
					v.drawScaled((12*FRACUNIT), (10*FRACUNIT), 3*FRACUNIT/4, p_gscore, V_SNAPTORIGHT|V_SNAPTOTOP|V_PERPLAYER|V_HUDTRANS, v.getColormap("SKINCOLOR_NONE", player.mo.color))
					v.drawNum(74, 11, player.score, V_SNAPTOTOP|V_SNAPTOLEFT|V_PERPLAYER|V_HUDTRANS)
					
			end
		end
	end
hud.add(shithud)

addHook("ThinkFrame", do
	for player in players.iterate
		if player.mo and player.mo.skin == "modernsonic"
			if player.startimerone == nil
			or player.startimerone > 12
				player.startimerone = 0
			end
			if player.startimertwo == nil
			or player.startimertwo > 18
				player.startimertwo = 0
			end
			if player.startimerthree == nil
			or player.startimerthree > 56
				player.startimerthree = 0
			end
			if player.flashingtimer == nil
			or player.flashingtimer > 15
				player.flashingtimer = 0
			end
			if player.powers[pw_super]
			or player.infntebst > 0
				player.flashingtimer = $1+1
			else
				player.flashingtimer = 0
			end
			if player.hudstyle == "Rush"
				if player.powers[pw_super]
					player.startimerthree = $1+8
				else
					if player.boostmeter >= 117
						if player.infntebst > 8*TICRATE
						or player.exiting
						or player.pflags & PF_FINISHED
							player.startimerthree = $1+8
						elseif player.infntebst > 5*TICRATE
							player.startimerthree = $1+4
						elseif player.infntebst > 2*TICRATE
							player.startimerthree = $1+2
						elseif player.infntebst > 0
							player.startimerthree = $1+1
						end
						player.startimertwo = 0
						player.startimerone = 0
					elseif player.boostmeter > 78
						player.startimerthree = 0
						player.startimertwo = $1+1
						player.startimerone = 0
					elseif player.boostmeter > 39
						player.startimerthree = 0
						player.startimertwo = 0
						player.startimerone = $1+1
					else
						player.startimerthree = 0
						player.startimertwo = 0
						player.startimerone = 0
					end
				end
			else
				player.startimerthree = 0
				player.startimertwo = 0
				player.startimerone = 0
			end
		end
	end
end)
			
			
			